<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output encoding="ISO-8859-1" indent="yes" method="xml" />
    <!--
        Generates the skeleton html for the screen, lays out the tables that make up the page calling appropriate templates to fill in page content.
    -->
    <xsl:template match="/" name="hyfinity_com_skin_html">
        <html xmlns="">
            <head>
                <!-- Fill in the contents of the <head> html element -->
                <xsl:call-template name="head_html" />
                <!-- Import the validatin script files -->
                <xsl:call-template name="validation_scripts" />
                <!-- Fill in the client side scripts -->
                <xsl:call-template name="page_scripts" />
            </head>
            <xsl:call-template name="body" />
        </html>
    </xsl:template>
    <xsl:template name="body">
        <body xmlns="">
            <!-- Main Screen Sections -->
            <!-- Fill in the main body -->
            <xsl:call-template name="main_body" />
            <!-- Fill in the contents of the page header, i.e. logo -->
            <xsl:call-template name="header" />
            <!-- Fill in the content of the top menu -->
            <div id="top_menu_primary">
                <xsl:call-template name="top_menu" />
            </div>
            <!-- tooltips container -->
            <div id="tipDiv" style="position:absolute; top:-100px; left:-300px; width : 300px; height : 100px; visibility:hidden;">
                <div class="messageTop" id="messageTop"></div>
                <iframe id="tipFrame" name="tipFrame" src="#" style="width :300px; height : 100px;" title="tipFrame"></iframe>
                <div id="messageDiv" onmouseout="outToolTip()" onmouseover="inToolTip()" style="position: absolute; top : 0 ; left : 0; width : 100%; height : auto;"></div>
                <div class="messageBottom" id="messageBottom"></div>
            </div>
        </body>
    </xsl:template>
<!--
        Place holder for client side scripts.
    -->
    <xsl:template name="page_scripts">
        <!-- Application specific script files are configured within FormMaker (AppMap screen)
             Do NOT add any here!!!!!!!! -->
    </xsl:template>
<!--
        Client side validation script files
    -->
    <xsl:template name="validation_scripts">
        <script type="text/javascript" xmlns="">
            djConfig={parseOnLoad: true};
        </script>
        <script src="../formmaker/js/dojo/dojo/dojo.js" type="text/javascript" xmlns="">//import dojo main script</script>
        <script src="js/DisplayUtils.js" type="text/javascript" xmlns="">//handles hide/display operations</script>
        <script src="js/FormValidator.js" type="text/javascript" xmlns="">//main entry point for validating a form</script>
        <script src="js/NumberValidator.js" type="text/javascript" xmlns="">//provides number validation</script>
        <script src="js/StringValidator.js" type="text/javascript" xmlns="">//provides string validation</script>
        <script src="js/BooleanValidator.js" type="text/javascript" xmlns="">//provides boolean validation</script>
        <script src="js/ValidationError.js" type="text/javascript" xmlns="">//represents a valiadtion error</script>
        <script src="js/ErrorDisplay.js" type="text/javascript" xmlns="">//shows validation errors on the form </script>
        <script src="js/DisplayMessages.js" type="text/javascript" xmlns="">//provides a list of display messages for each error</script>
        <script src="js/DateValidator.js" type="text/javascript" xmlns="">//provides date validation functions</script>
        <script src="js/FMActions.js" type="text/javascript" xmlns="">//provides implementations of the FM inbuilt actions</script>
        <script src="js/ValueConverter.js" type="text/javascript" xmlns="">//provides value conversion functions</script>
        <script src="js/combobox.js" type="text/javascript" xmlns="">//script required for editable combo box entries</script>
        <script src="js/date.js" type="text/javascript" xmlns="">//provides date manipulation functions</script>
        <script src="js/AnchorPosition.js" type="text/javascript" xmlns="">//determines the coordinates of an anchor</script>
        <script src="js/PopupWindow.js" type="text/javascript" xmlns="">//shows a popup window</script>
        <script src="js/CalendarPopup.js" type="text/javascript" xmlns="">//creates a calendar popup - needs the above two</script>
        <script type="text/javascript">
            function topLevelTabClicked(type)
            {
                var f = hyf.FMAction.getFormValidator().getForm();

                f.action = 'init.do';

                if (typeof(f.tab_control) != 'undefined')
                    f.tab_control.value = type;
                else
                    f.action += '?tab_control=' + type;

                f.submit();
            }
        </script>
    </xsl:template>
<!--
        Provides the contents for the HTML <head> element
    -->
    <xsl:template name="head_html">
        <meta content="text/html; charset=UTF-8" http-equiv="Content-Type" xmlns="" />
        <title xmlns="">
            <xsl:call-template name="page-title" />
        </title>
        <link href="images/favicon_soa_studio.ico" rel="shortcut icon" xmlns="" />
<!-- insert css -->
        <xsl:call-template name="css_imports" />
    </xsl:template>
    <xsl:template name="page-title">Deploy &amp; Publish - Hyfinity® - SOA Development Simplified</xsl:template>
<!--
        Placeholder for specifying css files for each page
    -->
    <xsl:template name="css_imports">
        <link href="css/hyfinity_com.css" rel="stylesheet" type="text/css" xmlns="" />
        <link href="css/hyfinity_apps.css" rel="stylesheet" type="text/css" xmlns="" />
        <link href="css/hyfinity_colours.css" rel="stylesheet" type="text/css" xmlns="" />
    </xsl:template>

    <xsl:template name="top_menu">
        <xsl:variable name="pageName">
            <xsl:call-template name="section_title" />
        </xsl:variable>
        <xsl:variable name="currentTab">
            <xsl:choose>
                <xsl:when test="$pageName = 'DeployResult' or $pageName = 'AdvancedDeploy'" >deployment</xsl:when>
                <xsl:otherwise>publication</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <div class="tab" xmlns="">
            <ul>
                <li>
                    <xsl:if test="$currentTab = 'deployment'" >
                        <xsl:attribute name="class">current</xsl:attribute>
                    </xsl:if>
                    <a href="javascript:topLevelTabClicked('deploy');" id="deploymentTab" name="deploymentTab">Deployment</a>
                </li>
                <li>
                    <xsl:if test="$currentTab = 'publication'" >
                        <xsl:attribute name="class">current</xsl:attribute>
                    </xsl:if>
                    <a href="javascript:topLevelTabClicked('publish');" id="publicationTab" name="publicationTab">Publication</a>
                </li>
                <li><a id="documentation" name="documentation" href="/documentation/index.html" target="_blank">Help...</a></li>
            </ul>
        </div>
    </xsl:template>


<!--
        Provides HTML content for the header at the top of the screen.
    -->
    <xsl:template name="header">
        <div class="tagline" xmlns="">
            <xsl:call-template name="tagline" />
        </div>
        <div class="logo" xmlns="">

        </div>
    </xsl:template>
<!--
        Provides HTML content for the tagline.
    -->
    <xsl:template name="tagline" xmlns:xg="http://www.hyfinity.com/xgate" xmlns:mvc="http://www.hyfinity.com/mvc" xmlns:xfact="http://www.hyfinity.com/xfactory">
        <a class="logout_btn" href="javascript:void(WinManager.confirmLogout(window));" id="logout" xmlns="">Change Workspace</a>
        <span class="heading" xmlns="">
            <xsl:if test="/mvc:eForm/mvc:Control/xg:project_info/xfact:project_details">
                <xsl:text>Project: </xsl:text>
                <xsl:value-of select="/mvc:eForm/mvc:Control/xg:project_info/xfact:project_details/xfact:project_name" />
            </xsl:if>
        </span>
    </xsl:template>

<!--
        Builds the main body of the page adding a header, footer and content area.
    -->
    <xsl:template name="main_body">
        <!-- Fill in the main body -->
        <div class="main_body" xmlns="">
            <xsl:call-template name="section_body" />
            <!--FIll in the contents of the page footer -->
            <xsl:call-template name="footer" />
        </div>
        <!-- Fill in the section title -->
        <div class="section_title" xmlns="">&#160;
        <!--<xsl:call-template name="section_title" />-->
        </div>
    </xsl:template>
    <!--
        Builds the section heading within main body.
    -->
    <xsl:template name="section_title">
        Section title will go here
    </xsl:template>
    <!--
        Builds the section heading within main body.
    -->
    <xsl:template name="section_body">
    </xsl:template>
<!--
        Provides HTML content for the footer.
    -->
    <xsl:template name="footer">
        <div class="footer" xmlns="">
            <xsl:text disable-output-escaping="yes">
                Copyright © 2001-2011 Hyfinity® Limited. All Rights Reserved.
            </xsl:text>
        </div>
    </xsl:template>
</xsl:stylesheet>
